import webbrowser

def Desc():
    return "This will open the URL in default browser"

async def Run(ctx, url: str, browser: str = 'default'):
    if browser == 'default':
        status = "Successfull" if webbrowser.open(url) else "Failed"
        await ctx.channel.send("The URL opening process was {}".format(status))
    else:
        c = webbrowser.get(browser)
        status = "Successfull" if c.open(url) else "Failed"
        await ctx.channel.send("The URL opening process was {}".format(status))


import os
import platform

def Desc():
    return "Show notifications with different types and levels"

async def Run(ctx, title: str, message: str, type: str = "messagebox", level: str = "info"):
    supported_types = ["messagebox", "toast"]
    supported_levels = ["error", "info", "warning"]

    if type not in supported_types:
        await ctx.channel.send("Unsupported notification type '{}'. Supported types are: {}".format(type, supported_types))
        return

    if level not in supported_levels:
        await ctx.channel.send("Unsupported notification level '{}'. Supported levels are: {}".format(level, supported_levels))
        return

    if platform.system() == 'Windows':
        if type == "messagebox":
            import ctypes
            ctypes.windll.user32.MessageBoxW(0, message, level.capitalize(), 0)
        elif type == "toast":
            from win10toast import ToastNotifier
            toaster = ToastNotifier()
            toaster.show_toast(title, message, duration=10)
    elif platform.system() == 'Linux':
        if type == "messagebox":
            os.system('zenity --{} --{} --text="{}"'.format(type, level, message))
        else:
            await ctx.channel.send("Unsupported notification type '{}' for Linux.".format(type))
    else:
        await ctx.channel.send("Unsupported operating system {}".format(platform.system()))


import serial
import pynmea2

rx_pin = 14
tx_pin = 15

def Desc():
    return "This will obtain the latest GPS information and share it here"

def read_gps_data():
    ser = serial.Serial("/dev/serial0", baudrate=9600, timeout=0.5)
    try:
        data = ser.readline().decode("utf-8")
        if "GPGGA" in data:
            msg = pynmea2.parse(data)
            return "https://www.google.com/maps/place/{},{}".format(msg.latitude, msg.longitude)
        return "Cannot reliability evaluate GPS data"

    except pynmea2.ParseError as e:
        return str(e)
    except Exception as e:
        return str(e)

async def Run(ctx):
    ctx.channel.send(read_gps_data())


import platform
import ctypes
import psutil
import os

def Desc():
    return "This will kill the active/specific running process"

async def Run(ctx, app_title=None):
    operating_system = platform.system()

    if operating_system == "Windows":
        if app_title:
            for process in psutil.process_iter(['pid', 'name']):
                if app_title.lower() in process.info['name'].lower():
                    try:
                        os.kill(process.info['pid'], 9)
                    except Exception as e:
                        print(f"Error killing process {process.info['pid']}: {e}")
            await ctx.channel.send(f"All instances of {app_title} killed.")
        else:
            ctypes.windll.user32.PostMessageW(ctypes.windll.user32.GetForegroundWindow(), 0x0112, 0xF060, 0)
            await ctx.channel.send("Default active window killed.")
    elif operating_system == "Linux":
        if app_title:
            for process in psutil.process_iter(['pid', 'name', 'cmdline']):
                if app_title.lower() in ' '.join(process.info['cmdline']).lower():
                    try:
                        process.terminate()
                    except Exception as e:
                        print(f"Error killing process {process.info['pid']}: {e}")
            await ctx.channel.send(f"All instances of {app_title} killed on Linux.")
        else:
            await ctx.channel.send("Killing the default active window not implemented on Linux yet.")
    else:
        await ctx.channel.send(f"Killing not supported on {operating_system}.")


def Desc():
    return "To clear all or x last messages from channel"

async def Run(ctx, count: int = 0):
    try:
        await ctx.message.delete()
        if count == 0:
            deleted = await ctx.channel.purge()
            await ctx.channel.send(f'Cleared all messages in the channel. ({len(deleted)} messages)')
        else:
            deleted = await ctx.channel.purge(limit=count)
            await ctx.channel.send(f'Cleared {len(deleted)} messages.')
    except Exception as e:
        await ctx.channel.send(str(e))


import subprocess

def Desc():
    return "To execute a terminal command and send the output"

async def Run(ctx, command: str):
    try:
        result = subprocess.check_output(command, shell=True, text=True)
        await ctx.channel.send(result)
    except subprocess.CalledProcessError as e:
        await ctx.channel.send(e.output)
    except Exception as e:
        await ctx.channel.send(f"Error executing command: {e}")


import os
import platform

def Desc():
    return "To speak the text using TTS"

async def Run(ctx, phrase: str):
    system_platform = platform.system()

    if system_platform == "Windows":
        await ctx.channel.send("Saying: " + phrase)
        os.system("powershell Add-Type -AssemblyName System.Speech; $synth = New-Object -TypeName System.Speech.Synthesis.SpeechSynthesizer; $synth.Speak('{}')".format(phrase))
    elif system_platform == "Linux":
        await ctx.channel.send("Saying: " + phrase)
        os.system('spd-say "{}"'.format(phrase))
    else:
        await ctx.channel.send("Can't use TTS unknown platform {}".format(system_platform))


import subprocess
import platform

def Desc():
    return "To Reboot the machine"

async def Run(ctx, delay: int = 0):
    system_platform = platform.system()

    if system_platform == "Windows":
        command = f"shutdown /r /t {delay}"
        await ctx.channel.send(f"Rebooting your Windows OS in {delay} seconds")
    elif system_platform == "Linux" or system_platform == "Darwin":
        command = f"shutdown -r {'now' if delay == 0 else f'+{delay}'}"
        await ctx.channel.send(f"Rebooting your Linux/MAC OS in {delay} seconds")
    else:
        await ctx.channel.send(f"Unsupported operating system: {system_platform}")
        return

    subprocess.run(command, shell=True)


import platform

def Desc():
    return "To control the volume of the PC range(0 - 100)"

async def Run(ctx, volume_level: int):
    try:
        system_platform = platform.system()

        if system_platform == "Windows":
            await ctx.channel.send("Setting volume to: {}%".format(volume_level))
            set_volume_windows(volume_level)
        elif system_platform == "Linux":
            await ctx.channel.send("Setting volume to: {}%".format(volume_level))
            set_volume_linux(volume_level)
        else:
            await ctx.channel.send("Can't control volume, unknown platform {}".format(system_platform))
    except Exception as e:
        await ctx.channel.send(str(e))

def set_volume_windows(volume_level):
    from ctypes import cast, POINTER
    from comtypes import CLSCTX_ALL
    from pycaw.pycaw import AudioUtilities, IAudioEndpointVolume

    devices = AudioUtilities.GetSpeakers()
    interface = devices.Activate(IAudioEndpointVolume._iid_, CLSCTX_ALL, None)
    volume = cast(interface, POINTER(IAudioEndpointVolume))
    volume.SetMasterVolumeLevelScalar(volume_level / 100, None)

def set_volume_linux(volume_level):
    from pulsectl import Pulse

    with Pulse('set_volume_linux') as pulse:
        for sink in pulse.sink_list():
            pulse.volume_set_all_chans(sink, volume_level / 100)


import discord
import shutil
import os

def Desc():
    return "Let the bot upload the file as a attachment for you, If a path is directory it will be compressed"

async def Run(ctx, file_path: str):
    try:
        if os.path.isdir(file_path):
            shutil.make_archive(file_path, 'zip', file_path)
            zip_file_path = f'{file_path}.zip'
            fname = os.path.basename(zip_file_path)
            zip_file = discord.File(zip_file_path, filename=fname)

            await ctx.channel.send(file=zip_file)
            os.remove(zip_file_path)
        else:
            with open(file_path, 'rb') as file:
                fname = os.path.basename(file_path)
                file_to_send = discord.File(file, filename=fname)
                
                await ctx.channel.send(file=file_to_send)
    except FileNotFoundError:
        await ctx.channel.send(f'The file or directory "{file_path}" does not exist.')
    except Exception as e:
        await ctx.channel.send("Error occured {}".format(str(e)))


import socket

def Desc():
    return "This will echo back hostname of the machine to check available instances of running bot, helpful if running multiple bots in different machines under same token"

async def Run(ctx):
    await ctx.channel.send("Hello From ".format(socket.gethostname()))


import discord
import pyautogui
from io import BytesIO

def Desc():
    return "This will capture the Screenshot"

async def Run(ctx):
    screenshot = pyautogui.screenshot()

    screenshot_bytes = BytesIO()
    screenshot.save(screenshot_bytes, format='PNG')
    screenshot_bytes.seek(0)

    await ctx.channel.send("Screenshot:", file=discord.File(screenshot_bytes, filename='screenshot.png'))


import os
import platform
import subprocess

def Desc():
    return "This will hybernate your PC"

async def Run(ctx):
    system_platform = platform.system()
    response = await hibernate_machine(system_platform)
    await ctx.channel.send(response)

async def hibernate_machine(platform):
    if platform == 'Windows':
        os.system('shutdown /h /t 1')
        return "Hybernating windows machine"
    elif platform == 'Linux':
        subprocess.run(['sudo', 'systemctl', 'hibernate'])
        return "Hybernating Linux machine"
    else:
        return f"Hibernation is not supported on {platform}."


def Desc():
    return "This will terminate (exit) the discord PC bot"

async def Run(ctx):
    await ctx.channel.send("Closing the Agent now, no more commands will be processed")
    await ctx.bot.close()


import subprocess
import platform

def Desc():
    return "To put the machine to sleep"

async def Run(ctx, delay: int = 0):
    system_platform = platform.system()

    if system_platform == "Windows":
        command = "rundll32.exe powrprof.dll,SetSuspendState 0,1,0"
        await ctx.channel.send(f"Putting your Windows OS to sleep in {delay} seconds")
    elif system_platform == "Linux" or system_platform == "Darwin":
        command = "pmset sleepnow"
        await ctx.channel.send(f"Putting your Linux/MAC OS to sleep in {delay} seconds")
    else:
        await ctx.channel.send(f"Unsupported operating system: {system_platform}")
        return

    subprocess.run(command, shell=True)


import platform
import ctypes
import os

def Desc():
    return "This will lock the machine"

async def Run(ctx):
    operating_system = platform.system()

    if operating_system == "Windows":
        ctypes.windll.user32.LockWorkStation()
        await ctx.channel.send("Don't worry i am locked now")
    elif operating_system == "Linux":
        os.system("xdg-screensaver lock")
        await ctx.channel.send("I tried to lock myself, let's see if i am successfull")
    elif operating_system == "Darwin":
        await ctx.channel.send("I don't know how to lock myself on this OS")
    else:
        await ctx.channel.send("I don't know very much about this OS {}".format(operating_system))


import os
import platform

def Desc():
    return "This will produce beep sound in the system"

async def Run(ctx, frequency: int = 1000, duration: int = 1000):
    if platform.system() == 'Windows':
        import winsound
        winsound.Beep(frequency, duration)
    elif platform.system() == 'Linux':
        os.system('printf "\a"')
    else:
        await ctx.channel.send("Unsupported operating system {}".format(platform.system()))


import os
import sys
import socket
import discord
from pydoc import importfile
from dotenv import load_dotenv
from discord.ext import commands
from discord.ext.commands import Bot

class BotControls(commands.Cog):
    def __init__(self, bot):
        self.bot = bot

    @commands.command(description="This will enable the bot specified by unique Hostname of the bot machine or all bots if not specified")
    async def enable(ctx, hostname="all"):
        if hostname == socket.gethostname() or hostname == "all":
            count = HandleCommands()
            await ctx.channel.send(f"Activated {count} commands")

    @commands.command(description="This will disable the bot specified by unique Hostname of the bot machine or all bots if not specified")
    async def disable(ctx, hostname="all"):
        if hostname == socket.gethostname() or hostname == "all":
            count = HandleCommands(register=False)
            await ctx.channel.send(f"Disabled {count} commands")

load_dotenv(dotenv_path=os.path.join(os.path.dirname(__file__), ".env"))
intents = discord.Intents.all()

bot = Bot(description="A remote administration discord BOT to control your PC", command_prefix='#', intents=intents)

def HandleCommands(register=True):
    commands = os.path.join(os.path.dirname(__file__), "commands")
    scripts = [file for file in os.listdir(commands) if file.endswith('.py') and file != '__init__.py']
    count = 0
    for name in scripts:
        command = name.split('.')[0]
        if register and command not in [c.name for c in bot.commands]:
            script_module = importfile(os.path.join(commands, name))
            Instance = getattr(script_module, 'Run', None)

            if Instance and callable(Instance):
                desc = getattr(script_module, 'Desc', 'Not Available')
                bot.command(name=command, help=desc())(Instance)
                count += 1
        elif not register and command != "ping" and command in [c.name for c in bot.commands]:
            bot.remove_command(command)
            count += 1
    return count

def Init(token=os.environ.get('TOKEN')):
    HandleCommands()
    bot.run(token)

@bot.event
async def on_ready():
    greeting = f'Hello master I "{socket.gethostname()}" am ready to serve you'
    channels = list(bot.get_all_channels())
    if len(channels) > 0:
        await channels.pop().send(greeting)
    if not bot.get_cog("BotControls"):
        await bot.add_cog(BotControls(bot))

@bot.event
async def on_disconnect():
    print("Connection broken from server")

@bot.event
async def on_error(event, *args, **kwargs):
    print(f"Error in event {event}: {sys.exc_info()} with ARGS {args} and {kwargs}")

@bot.event
async def on_command_error(event, *args, **kwargs):
    print(f"Error in command event {event}: {sys.exc_info()} with ARGS {args} and {kwargs}")

@bot.event
async def on_message(message):
    proceed = message.author != bot.user
    if proceed:
        await bot.process_commands(message)

def main():
    sys.path.append(os.path.dirname(__file__))
    if os.environ.get('TOKEN', None) is None:
        token = input("Enter Auth Token: ") if len(sys.argv) == 1 else sys.argv[1]
        with open(os.path.join(os.path.dirname(__file__), ".env"), 'w') as f:
            f.write(f'TOKEN={token}')
        Init(token)
    else:
        Init()

if __name__ == "__main__":
    main()


import psutil
import pygetwindow as gw
from tabulate import tabulate

def Desc():
    return "To get current system usage & status"

def get_cpu_usage():
    return psutil.cpu_percent(interval=1)

def get_ram_usage():
    return psutil.virtual_memory().percent

def get_active_window():
    active_window = gw.getActiveWindow()
    return active_window.title if active_window else 'N/A'

async def Run(ctx):
    response = [
        ['CPU', get_cpu_usage()],
        ['RAM', get_ram_usage()],
        ['Window', get_active_window()]
    ]

    message = tabulate(response, headers=['Resource', 'Usage'], tablefmt="pretty")
    await ctx.channel.send(message)


import discord
import pyaudio
import sounddevice as sd

class PyAudioPCM(discord.AudioSource):
    def __init__(self, channels=2, rate=48000, chunk=960, input_device=1) -> None:
        p = pyaudio.PyAudio()
        self.chunks = chunk
        self.input_stream = p.open(format=pyaudio.paInt16, channels=channels, rate=rate, input=True, input_device_index=input_device, frames_per_buffer=chunk)

    def read(self) -> bytes:
        return self.input_stream.read(self.chunks)

def Desc():
    return "To initiate a Voice call from Machine"

async def Run(ctx):
    if not ctx.author.voice:
        await ctx.channel.send("You need to first join the voice channel")
        return

    channel = ctx.author.voice.channel
    if ctx.voice_client is None:
        tunnel = await channel.connect()

        info = sd.query_devices(sd.default.device, 'input')
        source = PyAudioPCM(input_device=info['index'])
        tunnel.play(source, after=lambda e: print(f'Player error: {e}') if e else None)
    else:
        await ctx.voice_client.disconnect()


import discord
import cv2
from io import BytesIO
from PIL import Image

def Desc():
    return "This will capture a frame from the camera"

async def Run(ctx, camera: int = 0):
    cap = cv2.VideoCapture(camera)

    ret, frame = cap.read()

    if not ret:
        await ctx.channel.send("Failed to capture frame from the camera.")
        return

    pil_image = Image.fromarray(cv2.cvtColor(frame, cv2.COLOR_BGR2RGB))

    image_bytes = BytesIO()
    pil_image.save(image_bytes, format='PNG')
    image_bytes.seek(0)

    await ctx.channel.send("Camera Frame:", file=discord.File(image_bytes, filename='camera_frame.png'))

    cap.release()



import json
import discord
from tabulate import tabulate
from subprocess import check_output

def Desc():
    return "This will calculate & return the Current internet Bandwidth, Ping and other statistics"

def generate_map_url(lat, long):
    lat, long = float(lat), float(long)
    base_url = 'https://www.openstreetmap.org/export/embed.html?bbox='
    bbox = f'{long-0.005},{lat-0.005},{long+0.005},{lat+0.005}'
    return f'{base_url}{bbox}&layer=mapnik'

def convert_bytes(num):
    for x in ['bytes', 'KB', 'MB', 'GB', 'TB']:
        if num < 1024.0:
            return "%3.1f %s" % (num, x)
        num /= 1024.0

async def Run(ctx):
    try:
        await ctx.channel.send("Measuring internet statistics please wait")
        result = json.loads(check_output('speedtest-cli --json', shell=True, text=True))
        table = [
            ['Ping', "{} ms".format(result['ping'])],
            ['Download', convert_bytes(result['download'])],
            ['Upload', convert_bytes(result['upload'])],
            ['Server', result['server']['name']],
            ['IP', result['client']['ip']],
            ['ISP', result['client']['isp']],
            ['ISP Rating', result['client']['isprating']]
        ]
        table = tabulate(table, headers=['Attribute', 'Status'], tablefmt="pretty")
        if 'lat' in result['client'] and 'lon' in result['client']:
            url = generate_map_url(result['client']['lat'], result['client']['lon'])
            embed = discord.Embed(title="Location", description="Current Agent location wrt IP")
            embed.set_thumbnail(url=url)
            await ctx.channel.send(url, embed=embed)
        await ctx.channel.send(table)
    except Exception as e:
        await ctx.channel.send("Error: {}".format(e))